@@ -19,7 +19,9 @@ __version__ = "$Revision: 1.75 $"[10:-1]
 import __version__
 f2py_version = __version__.version
 
-import types,sys,copy,os
+import types
+import sys
+import copy
 errmess=sys.stderr.write
 
 ##################### Definitions ##################
@@ -228,6 +230,12 @@ cppmacros['PRINTPYOBJERR']="""\
 \tfprintf(stderr,\"\\n\");
 """
 cppmacros['MINMAX']="""\
+#ifndef max
+#define max(a,b) ((a > b) ? (a) : (b))
+#endif
+#ifndef min
+#define min(a,b) ((a < b) ? (a) : (b))
+#endif
 #ifndef MAX
 #define MAX(a,b) ((a > b) ? (a) : (b))
 #endif
@@ -468,15 +476,17 @@ cppmacros['CHECKARRAY']="""\
 cppmacros['CHECKSTRING']="""\
 #define CHECKSTRING(check,tcheck,name,show,var)\\
 \tif (!(check)) {\\
-\t\tPyErr_SetString(#modulename#_error,\"(\"tcheck\") failed for \"name);\\
-\t\tfprintf(stderr,show\"\\n\",slen(var),var);\\
+\t\tchar errstring[256];\\
+\t\tsprintf(errstring, \"%s: \"show, \"(\"tcheck\") failed for \"name, slen(var), var);\\
+\t\tPyErr_SetString(#modulename#_error, errstring);\\
 \t\t/*goto capi_fail;*/\\
 \t} else """
 cppmacros['CHECKSCALAR']="""\
 #define CHECKSCALAR(check,tcheck,name,show,var)\\
 \tif (!(check)) {\\
-\t\tPyErr_SetString(#modulename#_error,\"(\"tcheck\") failed for \"name);\\
-\t\tfprintf(stderr,show\"\\n\",var);\\
+\t\tchar errstring[256];\\
+\t\tsprintf(errstring, \"%s: \"show, \"(\"tcheck\") failed for \"name, var);\\
+\t\tPyErr_SetString(#modulename#_error,errstring);\\
 \t\t/*goto capi_fail;*/\\
 \t} else """
 ## cppmacros['CHECKDIMS']="""\
@@ -757,7 +767,7 @@ static int long_double_from_pyobj(long_double* v,PyObject *obj,const char *errme
 \t\t\treturn 1;
 \t\t}
 \t\telse if (PyArray_Check(obj) && PyArray_TYPE(obj)==PyArray_LONGDOUBLE) {
-\t\t\t(*v) = *((npy_longdouble *)PyArray_DATA(obj))
+\t\t\t(*v) = *((npy_longdouble *)PyArray_DATA(obj));
 \t\t\treturn 1;
 \t\t}
 \t}
@@ -1067,28 +1077,39 @@ def append_needs(need,flag=1):
             append_needs(n,flag)
     elif type(need)==types.StringType:
         if not need: return
-        if includes0.has_key(need): n = 'includes0'
-        elif includes.has_key(need): n = 'includes'
-        elif typedefs.has_key(need): n = 'typedefs'
-        elif typedefs_generated.has_key(need): n = 'typedefs_generated'
-        elif cppmacros.has_key(need): n = 'cppmacros'
-        elif cfuncs.has_key(need): n = 'cfuncs'
-        elif callbacks.has_key(need): n = 'callbacks'
-        elif f90modhooks.has_key(need): n = 'f90modhooks'
-        elif commonhooks.has_key(need): n = 'commonhooks'
+        if need in includes0:
+            n = 'includes0'
+        elif need in includes:
+            n = 'includes'
+        elif need in typedefs:
+            n = 'typedefs'
+        elif need in typedefs_generated:
+            n = 'typedefs_generated'
+        elif need in cppmacros:
+            n = 'cppmacros'
+        elif need in cfuncs:
+            n = 'cfuncs'
+        elif need in callbacks:
+            n = 'callbacks'
+        elif need in f90modhooks:
+            n = 'f90modhooks'
+        elif need in commonhooks:
+            n = 'commonhooks'
         else:
             errmess('append_needs: unknown need %s\n'%(`need`))
             return
         if need in outneeds[n]: return
         if flag:
             tmp={}
-            if needs.has_key(need):
+            if need in needs:
                 for nn in needs[need]:
                     t=append_needs(nn,0)
                     if type(t)==types.DictType:
                         for nnn in t.keys():
-                            if tmp.has_key(nnn): tmp[nnn]=tmp[nnn]+t[nnn]
-                            else: tmp[nnn]=t[nnn]
+                            if nnn in tmp:
+                                tmp[nnn]=tmp[nnn]+t[nnn]
+                            else:
+                                tmp[nnn]=t[nnn]
             for nn in tmp.keys():
                 for nnn in tmp[nn]:
                     if nnn not in outneeds[nn]:
@@ -1096,14 +1117,17 @@ def append_needs(need,flag=1):
             outneeds[n].append(need)
         else:
             tmp={}
-            if needs.has_key(need):
+            if need in needs:
                 for nn in needs[need]:
                     t=append_needs(nn,flag)
                     if type(t)==types.DictType:
                         for nnn in t.keys():
-                            if tmp.has_key(nnn): tmp[nnn]=t[nnn]+tmp[nnn]
-                            else: tmp[nnn]=t[nnn]
-            if not tmp.has_key(n): tmp[n]=[]
+                            if nnn in tmp:
+                                tmp[nnn]=t[nnn]+tmp[nnn]
+                            else:
+                                tmp[nnn]=t[nnn]
+            if n not in tmp:
+                tmp[n]=[]
             tmp[n].append(need)
             return tmp
     else:
@@ -1116,7 +1140,7 @@ def get_needs():
         out=[]
         saveout=copy.copy(outneeds[n])
         while len(outneeds[n])>0:
-            if not needs.has_key(outneeds[n][0]):
+            if outneeds[n][0] not in needs:
                 out.append(outneeds[n][0])
                 del outneeds[n][0]
             else:
@@ -1136,6 +1160,7 @@ def get_needs():
                 out=out+saveout
                 break
             saveout=copy.copy(outneeds[n])
-        if out==[]: out=[n]
+        if out==[]:
+            out=[n]
         res[n]=out
     return res
