@@ -1,10 +1,10 @@
-import sys
 import warnings
 
 from numpy.testing import *
 import numpy.lib
 from numpy.lib import *
 from numpy.core import *
+from numpy import matrix, asmatrix
 
 class TestAny(TestCase):
     def test_basic(self):
@@ -134,42 +134,6 @@ class TestSelect(TestCase):
         assert_equal(len(choices),3)
         assert_equal(len(conditions),3)
 
-class TestLogspace(TestCase):
-    def test_basic(self):
-        y = logspace(0,6)
-        assert(len(y)==50)
-        y = logspace(0,6,num=100)
-        assert(y[-1] == 10**6)
-        y = logspace(0,6,endpoint=0)
-        assert(y[-1] < 10**6)
-        y = logspace(0,6,num=7)
-        assert_array_equal(y,[1,10,100,1e3,1e4,1e5,1e6])
-
-class TestLinspace(TestCase):
-    def test_basic(self):
-        y = linspace(0,10)
-        assert(len(y)==50)
-        y = linspace(2,10,num=100)
-        assert(y[-1] == 10)
-        y = linspace(2,10,endpoint=0)
-        assert(y[-1] < 10)
-        y,st = linspace(2,10,retstep=1)
-        assert_almost_equal(st,8/49.0)
-        assert_array_almost_equal(y,mgrid[2:10:50j],13)
-
-    def test_corner(self):
-        y = list(linspace(0,1,1))
-        assert y == [0.0], y
-        y = list(linspace(0,1,2.5))
-        assert y == [0.0, 1.0]
-
-    def test_type(self):
-        t1 = linspace(0,1,0).dtype
-        t2 = linspace(0,1,1).dtype
-        t3 = linspace(0,1,2).dtype
-        assert_equal(t1, t2)
-        assert_equal(t2, t3)
-
 class TestInsert(TestCase):
     def test_basic(self):
         a = [1,2,3]
@@ -431,6 +395,44 @@ class TestTrapz(TestCase):
         #check integral of normal equals 1
         assert_almost_equal(sum(r,axis=0),1,7)
 
+    def test_ndim(self):
+        x = linspace(0, 1, 3)
+        y = linspace(0, 2, 8)
+        z = linspace(0, 3, 13)
+
+        wx = ones_like(x) * (x[1]-x[0])
+        wx[0] /= 2
+        wx[-1] /= 2
+        wy = ones_like(y) * (y[1]-y[0])
+        wy[0] /= 2
+        wy[-1] /= 2
+        wz = ones_like(z) * (z[1]-z[0])
+        wz[0] /= 2
+        wz[-1] /= 2
+
+        q = x[:,None,None] + y[None,:,None] + z[None,None,:]
+
+        qx = (q*wx[:,None,None]).sum(axis=0)
+        qy = (q*wy[None,:,None]).sum(axis=1)
+        qz = (q*wz[None,None,:]).sum(axis=2)
+
+        # n-d `x`
+        r = trapz(q, x=x[:,None,None], axis=0)
+        assert_almost_equal(r, qx)
+        r = trapz(q, x=y[None,:,None], axis=1)
+        assert_almost_equal(r, qy)
+        r = trapz(q, x=z[None,None,:], axis=2)
+        assert_almost_equal(r, qz)
+
+        # 1-d `x`
+        r = trapz(q, x=x, axis=0)
+        assert_almost_equal(r, qx)
+        r = trapz(q, x=y, axis=1)
+        assert_almost_equal(r, qy)
+        r = trapz(q, x=z, axis=2)
+        assert_almost_equal(r, qz)
+
+
 class TestSinc(TestCase):
     def test_simple(self):
         assert(sinc(0)==1)
@@ -440,7 +442,7 @@ class TestSinc(TestCase):
 
 class TestHistogram(TestCase):
     def setUp(self):
-        warnings.simplefilter('ignore', Warning)
+        warnings.simplefilter('ignore', DeprecationWarning)
 
     def tearDown(self):
         warnings.resetwarnings()
@@ -717,6 +719,10 @@ class TestNaNFuncts(TestCase):
                                    [ 0.59541557,  0.87964135,  0.70543747],
                                    [ 0.91084584,  0.84386844,  0.37068164]]))
 
+    def test_nanmin_allnan_on_axis(self):
+        assert_array_equal(isnan(nanmin([[nan]*2]*3, axis=1)),
+                     [True, True, True])
+
 
 class TestCorrCoef(TestCase):
     def test_simple(self):
@@ -776,6 +782,9 @@ class TestKaiser(TestCase):
                             array([ 0.58285404,  0.88409679,  1.        ,
                                     0.88409679,  0.58285404]))
 
+    def test_int_beta(self):
+        kaiser(3, 4)
+
 class TestMsort(TestCase):
     def test_simple(self):
         A = array([[ 0.44567325,  0.79115165,  0.5490053 ],
